import { world, ItemStack } from "@minecraft/server";

/* 
Manages if an entity hits another one
*/

export function entityHitEntityController(event) {
  const typeId = event.damagingEntity.typeId;
  if (typeId !== "minecraft:player") {
    return;
  } else {
    if (event.hitEntity.typeId.includes("me_gfts") && event.hitEntity.typeId.includes("gift")) {
      let giftState = event.hitEntity.getProperty("me_gfts:gift_state");
      if (giftState == "closed_empty") {
        if (event.hitEntity.getProperty("me_gfts:despawn_tick") < 4) {
          event.hitEntity.runCommand("/playanimation @s animation.me_gfts.gift.place_item");
          event.hitEntity.runCommand("/event entity @s me_gfts:despawn_tick_hit");
        }
        if (event.hitEntity.getProperty("me_gfts:despawn_tick") >= 4) {
          let giftColor = event.hitEntity.getProperty("me_gfts:color");
          let spawnedItem = new ItemStack("me_gfts:" + giftColor.toString() + "_gift");
          world.getDimension(event.damagingEntity.dimension.id).spawnItem(spawnedItem, event.hitEntity.location);
          event.damagingEntity.playSound("me_gfts_despawn_gift", { location: event.damagingEntity.location, pitch: 1, volume: 1 });

          event.hitEntity.runCommand("/setblock ~~~ air");
          event.hitEntity.runCommand("/event entity @s me_gfts:despawn_timer");
        }
      } else if (giftState == "closed_full") {
        event.hitEntity.runCommand("/playanimation @s animation.me_gfts.gift.place_item");
        event.damagingEntity.runCommand('/tellraw @s {"rawtext":[{"text":"§c[!] Interact to open the gift!"}]}');
        return;
      } else {
        return;
      }
    }
  }
}

function offsetPositionInDirection(position, viewDirection, offset) {
  const normalizedViewDirection = normalizeVector3(viewDirection);
  const offsetX = normalizedViewDirection.x * offset;
  const offsetY = normalizedViewDirection.y * offset;
  const offsetZ = normalizedViewDirection.z * offset;
  const offsetPosition = {
    x: position.x + offsetX,
    y: position.y + offsetY,
    z: position.z + offsetZ,
  };

  return offsetPosition;
}

function normalizeVector3(vector) {
  const length = Math.sqrt(vector.x * vector.x + vector.y * vector.y + vector.z * vector.z);
  if (length !== 0) {
    return {
      x: vector.x / length,
      y: vector.y / length,
      z: vector.z / length,
    };
  } else {
    return {
      x: 0,
      y: 0,
      z: 0,
    };
  }
}
