import { world, ItemStack } from "@minecraft/server";

const searchedBlocks = new Array("me_gfts:gift_block");

export function giftHandler() {
  const GiftsOverworld = world.getDimension("overworld").getEntities({
    type: "me_gfts:gift",
  });
  GiftsOverworld.forEach((gift) => {
    if (gift.isValid()) {
      handleGift(gift, "overworld");
    }
  });
  const GiftsNether = world.getDimension("nether").getEntities({
    type: "me_gfts:gift",
  });
  GiftsNether.forEach((gift) => {
    if (gift.isValid()) {
      handleGift(gift, "nether");
    }
  });
  const GiftsEnd = world.getDimension("the_end").getEntities({
    type: "me_gfts:gift",
  });
  GiftsEnd.forEach((gift) => {
    if (gift.isValid()) {
      handleGift(gift, "the_end");
    }
  });
}

function handleGift(gift, dimension) {
  if (gift.hasTag("me_gfts:spawn_item")) {
    let allGiftTags = gift.getTags();
    let giftingPlayerTag = getStringFromArray(allGiftTags, "me_gfts:gifting_player_");
    gift.runCommand('/tellraw @a[r=10] {"rawtext":[{"text":"§a§lCONGRATS! §r§fYou received a gift from §a' + getStringAfterSpecificPart(giftingPlayerTag, "me_gfts:gifting_player_") + '§f!"}]}');
    gift.runCommand("/setblock ~~~ air");
    let { container } = gift.getComponent("inventory");
    const item = container.getItem(0);
    world.getDimension(dimension).spawnItem(item, gift.location);
    gift.removeTag("me_gfts:spawn_item");
    // gift.remove();
  }
}

export function detectBlock(player) {
  const size = 6;
  const offset = Math.ceil(size / 2);
  for (const permutation of searchedBlocks) {
    for (let x = -offset; x < size - offset; x++) {
      for (let y = -offset; y < size - offset; y++) {
        for (let z = -offset; z < size - offset; z++) {
          const block = player.dimension.getBlock({ x: x + player.location.x, y: y + player.location.y, z: z + player.location.z });
          if (block === undefined) {
            continue;
          }
          if (block.permutation.matches(permutation)) {
            var nearbyGift = world.getDimension(player.dimension.id).getEntities({
              families: ["me_gfts_gift"],
              location: { x: x + player.location.x, y: y + player.location.y, z: z + player.location.z },
              maxDistance: 1,
              closest: 1,
            });
            if (nearbyGift.length === 0 || nearbyGift[0] == undefined) {
              player.runCommand("/setblock " + (x + player.location.x) + " " + (y + player.location.y) + " " + (z + player.location.z) + " air");
            }
          }
        }
      }
    }
  }
}




function getRandomIndex(max) {
  return Math.floor(Math.random() * max);
}

function getRandomInt(min, max) {
  // Ensure min and max are integers
  min = Math.ceil(min);
  max = Math.floor(max);
  // Generate random integer between min and max (inclusive)
  return Math.floor(Math.random() * (max - min + 1)) + min;
}

function getRandomElement(arr) {
  if (arr.length === 0) return undefined; // Handle empty array case
  const randomIndex = Math.floor(Math.random() * arr.length);
  return arr[randomIndex];
}

export function getStringFromArray(arry, string) {
  return arry.find((item) => item.startsWith(string));
}

function getStringAfterSpecificPart(originalString, specificString) {
  // Find the position of the specific string
  let startIndex = originalString.indexOf(specificString);

  // Check if the specific string exists in the original string
  if (startIndex !== -1) {
    // Return the part of the string after the specific string
    return originalString.substring(startIndex + specificString.length).trim();
  } else {
    // If the specific string is not found, return an empty string or a message
    return ""; // or you can return "Specific string not found"
  }
}
