import { system, world, ItemStack, StructureAnimationMode, StructureRotation, StructureSaveMode } from "@minecraft/server";
import { generateUUID } from "../events/uuid";
export function itemUseController(event) {
  const typeId = event.source.typeId;
  if (typeId !== "minecraft:player") {
    return;
  }
  const holdingItem = event.source.getComponent("equippable").getEquipment("Mainhand");
  if (holdingItem == undefined) {
    return;
  }
  if (holdingItem.typeId.includes("me_gfts") && holdingItem.typeId.includes("gift")) {
    const raycastBlock = event.source.getBlockFromViewDirection({ includeLiquidBlocks: false, includePassableBlocks: false, maxDistance: 8 });
    if (raycastBlock == undefined) {
      return;
    } else {
      const location = raycastBlock.block.location;
      const face = raycastBlock.face;
      let gift_name = "me_gfts:gift";
      let playerRotation = event.source.getRotation();
      let gift;

      let placeLoc;
      switch (face) {
        case "Up":
          placeLoc = { x: location.x + 0.5, y: location.y + 1, z: location.z + 0.5 };
          break;
        case "Down":
          placeLoc = { x: location.x + 0.5, y: location.y - 1, z: location.z + 0.5 };
          break;
        case "North":
          placeLoc = { x: location.x + 0.5, y: location.y, z: location.z - 0.5 };
          break;
        case "South":
          placeLoc = { x: location.x + 0.5, y: location.y, z: location.z + 1.5 };
          break;
        case "West":
          placeLoc = { x: location.x - 0.5, y: location.y, z: location.z + 0.5 };
          break;
        case "East":
          placeLoc = { x: location.x + 1.5, y: location.y, z: location.z + 0.5 };
          break;
        default:
          break;
      }

      if (checkforEntity(event, { x: placeLoc.x, y: placeLoc.y, z: placeLoc.z })) {
        return;
      }

      let giftStructureName = holdingItem.getDynamicProperty("me_gfts_structure_gift") ? holdingItem.getDynamicProperty("me_gfts_structure_gift") : "TEST EMPTY";

      // Old Camp is NOT saved on item aka new backpack/gift
      if (giftStructureName == "TEST EMPTY") {
        gift = world.getDimension(event.source.dimension.id).spawnEntity(gift_name, { x: location.x + 0.5, y: location.y + 1, z: location.z + 0.5 });
        gift.runCommand("/event entity @s me_gfts:" + getStringBetween(holdingItem.typeId, "me_gfts:", "_gift") + "_dye");
        event.source.runCommand("/setblock " + gift.location.x + " " + gift.location.y + " " + gift.location.z + " me_gfts:gift_block");
      } else {
        world.structureManager.place(giftStructureName, event.source.dimension, placeLoc, {
          animationMode: StructureAnimationMode.Layers,
          // animationSeconds: 0.1,
          rotation: StructureRotation.None,
        });
        let nearbyEntities = world.getDimension(event.source.dimension.id).getEntities({
          families: ["me_gfts_gift"],
          location: location.location,
          closest: 1,
        });
        if (nearbyEntities.length === 0 || nearbyEntities[0] == undefined) {
          return;
        } else {
          nearbyEntities[0].runCommand("/event entity @s me_gfts:" + getStringBetween(holdingItem.typeId, "me_gfts:", "_gift") + "_dye");
          event.source.runCommand("/setblock " + nearbyEntities[0].location.x + " " + nearbyEntities[0].location.y + " " + nearbyEntities[0].location.z + " me_gfts:gift_block");
          system.waitTicks(20).then(() => {
            world.structureManager.delete(giftStructureName);
          });
        }
      }

      event.source.runCommand("/clear @s " + holdingItem.typeId + " 0 1");
      return;
    }
  }
  // world.sendMessage(holdingItem.typeId);
  if (!event.source.isSneaking && holdingItem.typeId.includes("minecraft:") && holdingItem.typeId.includes("_dye")) {
    const raycastGift = event.source.getEntitiesFromViewDirection({ families: ["me_gfts_gift"], maxDistance: 6 })[0]?.entity;
    if (!raycastGift) {
      return;
    } else {
      let giftState = raycastGift.getProperty("me_gfts:gift_state");
      let giftColor = raycastGift.getProperty("me_gfts:color");
      if (giftState == "closed_empty") {
        if (giftColor == getStringBetween(holdingItem.typeId, "minecraft:", "_dye")) {
          event.source.runCommand('/tellraw @s {"rawtext":[{"text":"§c[!] The gift is already ' + getStringBetween(holdingItem.typeId, "minecraft:", "_dye").replace(/_/g, " ") + '!"}]}');
          return;
        }
        raycastGift.runCommand("/event entity @s me_gfts:" + getStringBetween(holdingItem.typeId, "minecraft:", "_dye") + "_dye");
        raycastGift.runCommand("/playsound me_gfts_dye_gift @a[r=30] ~~~ 1 1");
        raycastGift.runCommand("/particle me_gfts:dye_" + getStringBetween(holdingItem.typeId, "minecraft:", "_dye") + " ~~1~");
        event.source.runCommand("/clear @s " + holdingItem.typeId + " 0 1");
      } else if (giftState == "closed_full") {
        event.source.runCommand('/tellraw @s {"rawtext":[{"text":"§c[!] You cant dye while the gift is filled!"}]}');
      } else {
        // event.source.runCommand('/tellraw @s {"rawtext":[{"text":"§c[!] You cant dye while the gift is opening!"}]}');
      }
    }
  }

  if (!event.source.isSneaking && !(holdingItem.typeId.includes("me_gfts") && holdingItem.typeId.includes("gift")) && !(holdingItem.typeId.includes("minecraft:") && holdingItem.typeId.includes("_dye"))) {
    // world.sendMessage("interact with gift");
    const raycastGift = event.source.getEntitiesFromViewDirection({ families: ["me_gfts_gift"], maxDistance: 6 })[0]?.entity;
    if (!raycastGift) {
      return;
    } else {
      let giftState = raycastGift.getProperty("me_gfts:gift_state");
      if (giftState == "closed_empty") {
        event.source.runCommand('/tellraw @s {"rawtext":[{"text":"§c[!] Press SNEAK + INTERACT while holding an item to fill the gift!"}]}');
      }
    }
  }

  if (event.source.isSneaking && !(holdingItem.typeId.includes("me_gfts") && holdingItem.typeId.includes("gift"))) {
    // if (event.source.isSneaking && !(holdingItem.typeId.includes("minecraft:") && holdingItem.typeId.includes("_dye") && !(holdingItem.typeId.includes("me_gfts") && holdingItem.typeId.includes("gift")))) {

    // world.sendMessage("interact with gift");
    const raycastGift = event.source.getEntitiesFromViewDirection({ families: ["me_gfts_gift"], maxDistance: 6 })[0]?.entity;
    if (!raycastGift) {
      return;
    } else {
      let giftState = raycastGift.getProperty("me_gfts:gift_state");
      if (giftState == "closed_empty") {
        let { container } = raycastGift.getComponent("inventory");
        // const newholdingItem = new ItemStack(holdingItem.typeId, 1);
        container.setItem(0, holdingItem);
        event.source.runCommand("/replaceitem entity @s slot.weapon.mainhand 0 air ");
        raycastGift.runCommand("/playsound mob.chicken.plop @a[r=15] ~ ~ ~ 1 0.85");
        raycastGift.runCommand("/playanimation @s animation.me_gfts.gift.place_item");
        raycastGift.addTag("me_gfts:gifting_player_" + event.source.name);
        raycastGift.runCommand("/event entity @s me_gfts:closed_full");
      } else if (giftState == "opening" || giftState == "open_idle") {
        // event.source.runCommand('/tellraw @s {"rawtext":[{"text":"§c[!] You cant dye while the gift is opening!"}]}');
        return;
      } else {
        event.source.runCommand('/tellraw @s {"rawtext":[{"text":"§c[!] The gift is already filled!"}]}');
        return;
      }
    }
  }
}

function checkforEntity(event, blocklocation) {
  let closeEntity = world.getDimension(event.source.dimension.id).getEntities({
    location: blocklocation,
    maxDistance: 0.5,
    closest: 1,
  });
  if (closeEntity.length === 0 || closeEntity[0] == undefined) {
    return false;
  } else {
    event.source.runCommand('/tellraw @s {"rawtext":[{"text":"§c[!] You cant place the gift here, look for another location!"}]}');
    return true;
  }
}

function entityRotation(playerRotation) {
  if (playerRotation >= -22.5 && playerRotation <= 22.5) {
    // world.sendMessage(" 0");
    return 180;
  }
  if (playerRotation >= -67.5 && playerRotation <= -22.5) {
    // world.sendMessage("0");
    return 180;
  }
  if (playerRotation >= -112.5 && playerRotation <= -67.5) {
    // world.sendMessage("90");
    return 90;
  }
  if (playerRotation >= -157.5 && playerRotation <= -112.5) {
    // world.sendMessage("35");
    return 90;
  }
  if ((playerRotation >= 157.5 && playerRotation <= 180) || (playerRotation >= -180 && playerRotation <= -157.5)) {
    // world.sendMessage("180");
    return 0;
  }
  if (playerRotation >= 112.5 && playerRotation <= 157.5) {
    // world.sendMessage("225");
    return 270;
  }
  if (playerRotation >= 67.5 && playerRotation <= 112.5) {
    // world.sendMessage("270");
    return 270;
  }
  if (playerRotation >= 22.5 && playerRotation <= 67.5) {
    // world.sendMessage("315");
    return 180;
  }
}
function getStringBetween(str, start, end) {
  let startIndex = str.indexOf(start);
  if (startIndex === -1) {
    return ""; // Start string not found
  }

  startIndex += start.length; // Move past the start string

  let endIndex = str.indexOf(end, startIndex);
  if (endIndex === -1) {
    return ""; // End string not found
  }

  return str.substring(startIndex, endIndex);
}
