import { system, world, ItemStack, StructureAnimationMode, StructureRotation, StructureSaveMode } from "@minecraft/server";
import { playerJoins } from "./events/player_joins";
import { giftHandler, detectBlock } from "./events/gift_handler";
import { itemHold } from "./items/item_hold";
import { itemLore } from "./items/item_lore";
import { entityHitEntityController } from "./entity/entity_hit_entity";
import { itemUseController } from "./items/item_use_controller";

let curTick = 0;
const INIT_TIME = 0;

system.runTimeout(() => {
  system.runInterval(() => {
    try {
      if (curTick > INIT_TIME) {
        const allPlayers = world.getAllPlayers();
        allPlayers.forEach((player) => {
          if (!player.isValid()) return;
          else {
            itemHold(player);
            itemLore(player);
            playerJoins(player);
          }
        });

        giftHandler();
      }
      curTick++;
    } catch (e) {
      console.warn(e);
    }
  }, 0);
  system.runInterval(() => {
    try {
      if (curTick > INIT_TIME) {
        const allPlayers = world.getAllPlayers();
        allPlayers.forEach((player) => {
          if (!player.isValid()) return;
          else {
            detectBlock(player);
          }
        });
      }
      curTick++;
    } catch (e) {
      console.warn(e);
    }
  }, 30);
}, 130);

world.afterEvents.entityHitEntity.subscribe(entityHitEntityController);
world.afterEvents.itemUse.subscribe(itemUseController);

system.afterEvents.scriptEventReceive.subscribe((event) => {
  if (!(event.sourceEntity.typeId.includes("me_gfts") && event.sourceEntity.typeId.includes("gift"))) return;
  else {
    let gift = event.sourceEntity;
    switch (event.id) {
      case "me_gfts:save_gift":
        let structureUUID = "me_gfts:" + gift.id.toString();
        world.structureManager.delete(structureUUID);
        gift.runCommand("/playsound me_gfts_despawn_gift @a[r=10] ~~~ 1 1");
        system.waitTicks(10).then(() => {
          gift.runCommand("/setblock " + gift.location.x + " " + gift.location.y + " " + gift.location.z + " air");
          world.structureManager.createFromWorld(structureUUID, gift.dimension, gift.location, gift.location, { saveMode: StructureSaveMode.World, includeEntities: true });
          // world.getDimension(gift.dimension).playSound("me_gfts_despawn_gift", { location: gift.location, pitch: 1, volume: 1 });
          let giftColor = gift.getProperty("me_gfts:color");
          let spawnedItem = new ItemStack("me_gfts:" + giftColor.toString() + "_gift");
          spawnedItem.setDynamicProperty("me_gfts_structure_gift", structureUUID);
          world.getDimension(gift.dimension.id).spawnItem(spawnedItem, gift.location);
          gift.remove();
        });
        break;
      default:
        break;
    }
  }
});
